dnl acx_fc_check_strprt_convert.m4 --- check F2003 string pointer conversion
dnl
dnl Copyright  (C)  2010  Thomas Jahns <jahns@dkrz.de>
dnl
dnl Version: 1.0
dnl Keywords:
dnl Author: Thomas Jahns <jahns@dkrz.de>
dnl Maintainer: Thomas Jahns <jahns@dkrz.de>
dnl URL: https://www.dkrz.de/redmine/projects/show/scales-ppm
dnl
dnl Redistribution and use in source and binary forms, with or without
dnl modification, are  permitted provided that the following conditions are
dnl met:
dnl
dnl Redistributions of source code must retain the above copyright notice,
dnl this list of conditions and the following disclaimer.
dnl
dnl Redistributions in binary form must reproduce the above copyright
dnl notice, this list of conditions and the following disclaimer in the
dnl documentation and/or other materials provided with the distribution.
dnl
dnl Neither the name of the DKRZ GmbH nor the names of its contributors
dnl may be used to endorse or promote products derived from this software
dnl without specific prior written permission.
dnl
dnl THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
dnl IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
dnl TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
dnl PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
dnl OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
dnl EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
dnl PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
dnl PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
dnl LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
dnl NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
dnl SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
dnl
dnl Commentary:
dnl
dnl Checks the Fortran compiler for features required to enable the Fortran
dnl 2003 interface. The Fortran code that is used for the test is based on the
dnl output of the following command:
dnl   ../interfaces/f2003/bindGen.rb header.h conftest_mod
dnl where header.h contains a single line:
dnl   const char* strerror(int errnum);
dnl The function strerror is a part of the C standard library, which
dnl executables generated by any modern Fortran compiler are automatically
dnl linked to. Although the documented return value of the function is of
dnl char*, in fact, it returns a pointer to a constant. Therefore, we declare
dnl it with the const modifier, so the code generated by bindGen.rb would not
dnl try to free the memory referenced by the returned pointer once the output
dnl of the function is saved as a Fortran string.
dnl
dnl
dnl Code:
dnl
AC_DEFUN([ACX_FC_CHECK_STRPTR_CONVERT],
  [AC_LANG_PUSH([Fortran])
   AC_MSG_CHECKING([if Fortran compiler can handle complex CHARACTER interoperability])
   AC_COMPILE_IFELSE([AC_LANG_SOURCE([module conftest_mod
  use iso_c_binding
  implicit none
  private

  interface
    integer(c_size_t) function lib_strlen(charPtr) bind(c, name = "strlen")
      import c_size_t, c_ptr
      type(c_ptr), value :: charPtr
    end function lib_strlen
  end interface

  public :: strerror

contains

  function strerror(errnum_dummy) result(f_result)
    character(kind = c_char), dimension(:), pointer :: f_result
    integer(c_int), value :: errnum_dummy
    type(c_ptr) :: ptr
    integer :: rv_shape(1)
    interface
      function lib_strerror(errnum_dummy) bind(c, name = 'strerror')&
      & result(c_result)
        import c_int, c_ptr
        type(c_ptr) :: c_result
        integer(c_int), value :: errnum_dummy
      end function lib_strerror
    end interface
    f_result => null()
    ptr = lib_strerror(errnum_dummy)
    if(c_associated(ptr)) then
      rv_shape(1) = int(lib_strlen(ptr))
      call c_f_pointer(ptr, f_result, rv_shape)
    end if
  end function strerror

end module conftest_mod

program conftest
  use iso_c_binding
  use conftest_mod, only: strerror
  implicit none
  character(kind=c_char), dimension(:), pointer :: msgArr
  character(kind=c_char, len=:), allocatable :: msgStr
  integer :: i

  msgArr => strerror(42)
  allocate(character(kind=c_char, len=size(msgArr)) :: msgStr)
  forall (i = 1:len(msgStr)) msgStr(i:i) = msgArr(i)
  write (0, '(a)') msgStr
end program conftest])],
     [AC_MSG_RESULT([yes])
m4_ifval([$1],[$1])],
     [AC_MSG_RESULT([no])
m4_ifval([$2],[$2])])
   AC_LANG_POP([Fortran])])

dnl
dnl Local Variables:
dnl mode: autoconf
dnl End:
dnl
